// Copyright (c) 2021 Tencent. All rights reserved.
// Author: tatemin

import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:tencent_calls_engine/platform/tuicall_engine_platform_interface.dart';
import 'package:tencent_calls_engine/src/tuicall_observer.dart';
import 'package:tencent_calls_engine/src/tuicall_define.dart';
import 'package:tencent_calls_engine/src/tuicall_callback.dart';

/// An implementation of [TUICallEnginePlatform] that uses method channels.
class MethodChannelTUICallEngine extends TUICallEnginePlatform {
  final List<TUICallObserver> _observerList = [];
  @visibleForTesting
  static const methodChannel = MethodChannel('tuicall_engine');

  MethodChannelTUICallEngine() {
    methodChannel.setMethodCallHandler((call) async {
      if (call.method == "onTUICallObserver") {
        String observerName = call.arguments['method'];
        var observerParams = call.arguments['params'];
        _onTUICallObserver(observerName, observerParams);
      } else {
        debugPrint("onTUICallObserver: MethodNotImplemented ${call.arguments}");
      }
    });
  }

  @override
  Future<void> addObserver(TUICallObserver observer) async {
    if (_observerList.contains(observer)) {
      return;
    }
    _observerList.add(observer);
    methodChannel.invokeMethod("addObserver", {});
  }

  @override
  Future<void> removeObserver(TUICallObserver observer) async {
    _observerList.remove(observer);
    if (_observerList.isEmpty) {
      methodChannel.invokeMethod("removeObserver", {});
    }
  }

  @override
  Future<TUIResult> setVideoEncoderParams(VideoEncoderParams params) async {
    try {
      await methodChannel.invokeMethod("setVideoEncoderParams", {"params": params.toJson()});
    } on PlatformException catch (error) {
      return TUIResult(code: error.code, message: error.message);
    } on Exception catch (error) {
      return TUIResult(code: "-1", message: error.toString());
    }
    return TUIResult(code: "", message: "setVideoEncoderParams success");
  }

  @override
  Future<TUIResult> setVideoRenderParams(String userId, VideoRenderParams params) async {
    try {
      await methodChannel.invokeMethod("setVideoRenderParams", {"userId": userId, "params": params.toJson()});
    } on PlatformException catch (error) {
      return TUIResult(code: error.code, message: error.message);
    } on Exception catch (error) {
      return TUIResult(code: "-1", message: error.toString());
    }
    return TUIResult(code: "", message: "setVideoRenderParams success");
  }

  @override
  Future<TUIResult> hangup() async {
    try {
      await methodChannel.invokeMethod("hangup", {});
    } on PlatformException catch (error) {
      return TUIResult(code: error.code, message: error.message);
    } on Exception catch (error) {
      return TUIResult(code: "-1", message: error.toString());
    }
    return TUIResult(code: "", message: "hangup success");
  }

  @override
  Future<TUIResult> init(int sdkAppID, String userId, String userSig) async {
    try {
      await methodChannel.invokeMethod("init", {'sdkAppID': sdkAppID, 'userId': userId, 'userSig': userSig});
    } on PlatformException catch (error) {
      return TUIResult(code: error.code, message: error.message);
    } on Exception catch (error) {
      return TUIResult(code: "-1", message: error.toString());
    }
    return TUIResult(code: "", message: "init success");
  }

  @override
  Future<TUIResult> unInit() async {
    try {
      await methodChannel.invokeMethod("unInit", {});
    } on PlatformException catch (error) {
      return TUIResult(code: error.code, message: error.message);
    } on Exception catch (error) {
      return TUIResult(code: "-1", message: error.toString());
    }
    return TUIResult(code: "", message: "unInit success");
  }

  @override
  Future<TUIResult> call(String userId, TUICallMediaType mediaType, TUICallParams params) async {
    try {
      await methodChannel
          .invokeMethod("call", {'userId': userId, 'mediaType': mediaType.index, 'params': params.toJson()});
    } on PlatformException catch (error) {
      return TUIResult(code: error.code, message: error.message);
    } on Exception catch (error) {
      return TUIResult(code: "-1", message: error.toString());
    }
    return TUIResult(code: "", message: "call success");
  }

  @override
  Future<TUIResult> groupCall(
      String groupId, List<String> userIdList, TUICallMediaType mediaType, TUICallParams params) async {
    try {
      await methodChannel.invokeMethod("groupCall",
          {'groupId': groupId, 'userIdList': userIdList, 'mediaType': mediaType.index, 'params': params.toJson()});
    } on PlatformException catch (error) {
      return TUIResult(code: error.code, message: error.message);
    } on Exception catch (error) {
      return TUIResult(code: "-1", message: error.toString());
    }
    return TUIResult(code: "", message: "groupCall success");
  }

  @override
  Future<TUIResult> accept() async {
    try {
      await methodChannel.invokeMethod("accept", {});
    } on PlatformException catch (error) {
      return TUIResult(code: error.code, message: error.message);
    } on Exception catch (error) {
      return TUIResult(code: "-1", message: error.toString());
    }
    return TUIResult(code: "", message: "accept success");
  }

  @override
  Future<TUIResult> reject() async {
    try {
      await methodChannel.invokeMethod("reject", {});
    } on PlatformException catch (error) {
      return TUIResult(code: error.code, message: error.message);
    } on Exception catch (error) {
      return TUIResult(code: "-1", message: error.toString());
    }
    return TUIResult(code: "", message: "reject success");
  }

  @override
  Future<TUIResult> ignore() async {
    try {
      await methodChannel.invokeMethod("ignore", {});
    } on PlatformException catch (error) {
      return TUIResult(code: error.code, message: error.message);
    } on Exception catch (error) {
      return TUIResult(code: "-1", message: error.toString());
    }
    return TUIResult(code: "", message: "ignore success");
  }

  @override
  Future<void> iniviteUser(List<String> userIdList, TUICallParams params, TUIValueCallback callback) async {
    try {
      var userIds =
          await methodChannel.invokeMethod("iniviteUser", {'userIdList': userIdList, 'params': params.toJson()});
      callback.onSuccess!(userIds);
    } on PlatformException catch (error) {
      callback.onError!(int.fromEnvironment(error.code), error.message!);
    } on Exception catch (error) {
      callback.onError!(-1, error.toString());
    }
  }

  @override
  Future<TUIResult> joinInGroupCall(TUIRoomId roomId, String groupId, TUICallMediaType mediaType) async {
    try {
      await methodChannel.invokeMethod(
          "joinInGroupCall", {'roomId': roomId.toJson(), 'groupId': groupId, 'mediaType': mediaType.index});
    } on PlatformException catch (error) {
      return TUIResult(code: error.code, message: error.message);
    } on Exception catch (error) {
      return TUIResult(code: "-1", message: error.toString());
    }
    return TUIResult(code: "", message: "joinInGroupCall success");
  }

  @override
  Future<void> switchCallMediaType(TUICallMediaType mediaType) async {
    await methodChannel.invokeMethod("switchCallMediaType", {'mediaType': mediaType.index});
  }

  @override
  Future<void> startRemoteView(String userId, int? viewId) async {
    await methodChannel.invokeMethod("startRemoteView", {'userId': userId, 'viewId': viewId});
  }

  @override
  Future<void> stopRemoteView(String userId) async {
    await methodChannel.invokeMethod("stopRemoteView", {'userId': userId});
  }

  @override
  Future<TUIResult> openCamera(TUICamera camera, int? viewId) async {
    try {
      await methodChannel.invokeMethod("openCamera", {'camera': camera.index, 'viewId': viewId});
    } on PlatformException catch (error) {
      return TUIResult(code: error.code, message: error.message);
    } on Exception catch (error) {
      return TUIResult(code: "-1", message: error.toString());
    }
    return TUIResult(code: "", message: "openCamera success");
  }

  @override
  Future<void> closeCamera() async {
    await methodChannel.invokeMethod("closeCamera", {});
  }

  @override
  Future<void> switchCamera(TUICamera camera) async {
    await methodChannel.invokeMethod("switchCamera", {'camera': camera.index});
  }

  @override
  Future<TUIResult> openMicrophone() async {
    try {
      await methodChannel.invokeMethod("openMicrophone", {});
    } on PlatformException catch (error) {
      return TUIResult(code: error.code, message: error.message);
    } on Exception catch (error) {
      return TUIResult(code: "-1", message: error.toString());
    }
    return TUIResult(code: "", message: "openMicrophone success");
  }

  @override
  Future<void> closeMicrophone() async {
    await methodChannel.invokeMethod("closeMicrophone", {});
  }

  @override
  Future<void> selectAudioPlaybackDevice(TUIAudioPlaybackDevice device) async {
    await methodChannel.invokeMethod("selectAudioPlaybackDevice", {'device': device.index});
  }

  @override
  Future<TUIResult> setSelfInfo(String nickname, String avatar) async {
    try {
      await methodChannel.invokeMethod("setSelfInfo", {'nickname': nickname, 'avatar': avatar});
    } on PlatformException catch (error) {
      return TUIResult(code: error.code, message: error.message);
    } on Exception catch (error) {
      return TUIResult(code: "-1", message: error.toString());
    }
    return TUIResult(code: "", message: "setSelfInfo success");
  }

  @override
  Future<TUIResult> enableMultiDeviceAbility(bool enable) async {
    try {
      await methodChannel.invokeMethod("enableMultiDeviceAbility", {'enable': enable});
    } on PlatformException catch (error) {
      return TUIResult(code: error.code, message: error.message);
    } on Exception catch (error) {
      return TUIResult(code: "-1", message: error.toString());
    }
    return TUIResult(code: "", message: "enableMultiDeviceAbility success");
  }

  @override
  Future<void> queryRecentCalls(TUICallRecentCallsFilter filter, TUIValueCallback callback) async {
    try {
      var result = await methodChannel.invokeMethod("queryRecentCalls", {'filter': filter.toJson()});
      List<TUICallRecords> callRecords = [];
      final recordList = List<dynamic>.from(result);
      for (var recordMap in recordList) {
        TUICallRecords record = TUICallRecords();
        record.callId = recordMap['callId'];
        record.inviter = recordMap['inviter'];
        record.inviteList = List<String>.from(recordMap['inviteList']);
        record.scene = TUICallScene.values[recordMap['scene']];
        record.mediaType = TUICallMediaType.values[recordMap['mediaType']];
        record.groupId = recordMap['groupId'];
        record.role = TUICallRole.values[recordMap['role']];
        record.result = TUICallResultType.values[recordMap['result']];
        record.beginTime = recordMap['beginTime'];
        record.totalTime = recordMap['totalTime'];
        callRecords.add(record);
      }
      callback.onSuccess!(result);
    } on PlatformException catch (error) {
      callback.onError!(int.fromEnvironment(error.code), error.message!);
    } on Exception catch (error) {
      callback.onError!(-1, error.toString());
    }
  }

  @override
  Future<void> deleteRecordCalls(
    List<String> callIdList,
    TUIValueCallback callback,
  ) async {
    try {
      var callIds = await methodChannel.invokeMethod("deleteRecordCalls", {'callIdList': callIdList});
      callback.onSuccess!(callIds);
    } on PlatformException catch (error) {
      callback.onError!(int.fromEnvironment(error.code), error.message!);
    } on Exception catch (error) {
      callback.onError!(-1, error.toString());
    }
  }

  @override
  Future<TUIResult> setBeautyLevel(double level) async {
    try {
      await methodChannel.invokeMethod("setBeautyLevel", {'level': level});
    } on PlatformException catch (error) {
      return TUIResult(code: error.code, message: error.message);
    } on Exception catch (error) {
      return TUIResult(code: "-1", message: error.toString());
    }
    return TUIResult(code: "", message: "setBeautyLevel success");
  }

  @override
  Future<void> callExperimentalAPI(Map<String, dynamic> jsonMap) async {
    await methodChannel.invokeMethod("callExperimentalAPI", {'jsonMap': jsonMap});
  }

  void _onTUICallObserver(String observerName, var params) {
    for (var observer in _observerList) {
      switch (observerName) {
        case "onError":
          {
            observer.onError != null ? observer.onError!(params["code"], params['message']) : false;
            break;
          }

        case "onCallReceived":
          {
            final calleeIdList = List<String>.from(params['calleeIdList']);
            TUICallMediaType callMediaType = TUICallMediaType.values[params['callMediaType']];
            observer.onCallReceived != null
                ? observer.onCallReceived!(params['callerId'], calleeIdList, params['groupId'], callMediaType)
                : false;
            break;
          }

        case "onCallCancelled":
          {
            observer.onCallCancelled != null ? observer.onCallCancelled!(params['callerId']) : false;
            break;
          }

        case "onCallBegin":
          {
            final roomIdMap = Map<String, int>.from(params['roomId']);
            TUIRoomId roomId = TUIRoomId.intRoomId(intRoomId: roomIdMap['intRoomId']!);
            TUICallMediaType callMediaType = TUICallMediaType.values[params['callMediaType']];
            TUICallRole callRole = TUICallRole.values[params['callRole']];
            observer.onCallBegin != null ? observer.onCallBegin!(roomId, callMediaType, callRole) : false;
            break;
          }

        case "onCallEnd":
          {
            TUIRoomId roomId = TUIRoomId.intRoomId(intRoomId: params['roomId']['intRoomId']);
            TUICallMediaType callMediaType = TUICallMediaType.values[params['callMediaType']];
            TUICallRole callRole = TUICallRole.values[params['callRole']];
            double totalTime = params['totalTime'];
            observer.onCallEnd != null ? observer.onCallEnd!(roomId, callMediaType, callRole, totalTime) : false;
            break;
          }

        case "onCallMediaTypeChanged":
          {
            TUICallMediaType oldCallMediaType = TUICallMediaType.values[params['oldCallMediaType']];
            TUICallMediaType newCallMediaType = TUICallMediaType.values[params['newCallMediaType']];
            observer.onCallMediaTypeChanged != null
                ? observer.onCallMediaTypeChanged!(oldCallMediaType, newCallMediaType)
                : false;
            break;
          }

        case "onUserReject":
          {
            observer.onUserReject != null ? observer.onUserReject!(params['userId']) : false;
            break;
          }

        case "onUserNoResponse":
          {
            observer.onUserNoResponse != null ? observer.onUserNoResponse!(params['userId']) : false;
            break;
          }

        case "onUserLineBusy":
          {
            observer.onUserLineBusy != null ? observer.onUserLineBusy!(params['userId']) : false;
            break;
          }

        case "onUserJoin":
          {
            observer.onUserJoin != null ? observer.onUserJoin!(params['userId']) : false;
            break;
          }

        case "onUserLeave":
          {
            observer.onUserLeave != null ? observer.onUserLeave!(params['userId']) : false;
            break;
          }

        case "onUserVideoAvailable":
          {
            observer.onUserVideoAvailable != null
                ? observer.onUserVideoAvailable!(params['userId'], params['isVideoAvailable'])
                : false;
            break;
          }

        case "onUserAudioAvailable":
          {
            observer.onUserAudioAvailable != null
                ? observer.onUserAudioAvailable!(params['userId'], params['isAudioAvailable'])
                : false;
            break;
          }

        case "onUserVoiceVolumeChanged":
          {
            final volumeMap = Map<String, int>.from(params['volumeMap']);
            observer.onUserVoiceVolumeChanged != null ? observer.onUserVoiceVolumeChanged!(volumeMap) : false;
            break;
          }

        case "onUserNetworkQualityChanged":
          {
            List returnNetworkQualityList = params['networkQualityList'];
            List<TUINetworkQualityInfo> networkQualityLists = [];
            for (final element in returnNetworkQualityList) {
              var userId = (element['userId'] == null) ? "" : element['userId'];
              TUINetworkQuality quality = TUINetworkQuality.values[element["networkQuality"]];
              networkQualityLists.add(TUINetworkQualityInfo(userId: userId, quality: quality));
            }
            observer.onUserNetworkQualityChanged != null
                ? observer.onUserNetworkQualityChanged!(networkQualityLists)
                : false;
            break;
          }

        case "onKickedOffline":
          {
            observer.onKickedOffline != null ? observer.onKickedOffline!() : false;
            break;
          }

        case "onUserSigExpired":
          {
            observer.onUserSigExpired != null ? observer.onUserSigExpired!() : false;
            break;
          }
      }
    }
  }
}
