import 'package:plugin_platform_interface/plugin_platform_interface.dart';
import 'package:tencent_calls_engine/platform/tuicall_engine_platform_interface.dart';
import 'package:tencent_calls_engine/src/tuicall_callback.dart';
import 'package:tencent_calls_engine/src/tuicall_define.dart';
import 'package:tencent_calls_engine/src/tuicall_observer.dart';

class TUICallEngine extends PlatformInterface {
  TUICallEngine() : super(token: _token);

  static final Object _token = Object();

  static TUICallEngine _instance = TUICallEngine();

  static TUICallEngine get instance => _instance;

  static set instance(TUICallEngine instance) {
    PlatformInterface.verifyToken(instance, _token);
    _instance = instance;
  }

  /// 初始化SDK函数，请在使用所有功能之前先调用该函数，以便完成包含通话服务鉴权在内初始化动作。
  ///
  /// @param sdkAppID  应用ID
  /// @param userId    当前用户的 ID
  /// @param userSig   安全保护签名
  /// return TUIResult
  Future<TUIResult> init(int sdkAppID, String userId, String userSig) async {
    return await TUICallEnginePlatform.instance.init(sdkAppID, userId, userSig);
  }

  Future<TUIResult> unInit() async {
    return await TUICallEnginePlatform.instance.unInit();
  }

  Future<void> addObserver(TUICallObserver observer) async {
    await TUICallEnginePlatform.instance.addObserver(observer);
  }

  Future<void> removeObserver(TUICallObserver observer) async {
    await TUICallEnginePlatform.instance.removeObserver(observer);
  }

  /// Set the encoding parameters of the video encoder
  /// This setting can determine the image quality seen by the remote user,
  /// as well as the image quality of the video files recorded in the cloud.
  ///
  /// @param params   Encoding parameters: screen resolution, video aspect ratio mode
  /// @param callback result callback
  ///
  Future<TUIResult> setVideoEncoderParams(VideoEncoderParams params) async {
    return await TUICallEnginePlatform.instance.setVideoEncoderParams(params);
  }

  ///
  /// Set the rendering mode of the user screen
  ///
  /// @param userId   userid
  /// @param params   Screen rendering parameters: screen rotation angle, fill mode
  /// @param callback result callback
  ///
  Future<TUIResult> setVideoRenderParams(
      String userId, VideoRenderParams params) async {
    return await TUICallEnginePlatform.instance
        .setVideoRenderParams(userId, params);
  }

  ///
  /// Hangup
  ///
  /// @param callback result callback
  ///
  Future<TUIResult> hangup() async {
    return await TUICallEnginePlatform.instance.hangup();
  }

  /// 拨打电话（1v1通话）
  ///
  /// @param roomId        此次通话的音视频房间 ID
  /// @param userId        目标用户的 userId
  /// @param callMediaType 通话的媒体类型，例如：视频通话、语音通话
  /// @param params        通话参数扩展字段，例如：离线推送自定义内容
  /// return TUIResult
  Future<TUIResult> call(
      String userId, TUICallMediaType mediaType, TUICallParams params) async {
    return await TUICallEnginePlatform.instance.call(userId, mediaType, params);
  }

  /// 发起群组通话，注意：使用群组通话前需要创建IM 群组，如果已经创建，请忽略；
  ///
  /// @param roomId        此次通话的音视频房间 ID
  /// @param groupId       此次群组通话的群 ID
  /// @param userIdList    目标用户的 userId 列表
  /// @param callMediaType 通话的媒体类型，例如：视频通话、语音通话
  /// @param params        通话参数扩展字段，例如：离线推送自定义内容
  /// return TUIResult
  Future<TUIResult> groupCall(String groupId, List<String> userIdList,
      TUICallMediaType mediaType, TUICallParams params) async {
    return await TUICallEnginePlatform.instance
        .groupCall(groupId, userIdList, mediaType, params);
  }

  /// 接受当前通话，当您作为被叫收到 onCallReceived() 的回调时，可以调用该函数接听来电。
  ///
  /// return TUIResult
  Future<TUIResult> accept() async {
    return await TUICallEnginePlatform.instance.accept();
  }

  /// 拒绝当前通话，当您作为被叫收到 onCallReceived() 的回调时，可以调用该函数拒绝来电。
  ///
  /// return TUIResult
  Future<TUIResult> reject() async {
    return await TUICallEnginePlatform.instance.reject();
  }

  /// 忽略当前通话，当您作为被叫收到 onCallReceived() 的回调时，可以调用该函数忽略来电，此时主叫会收到 onUserLineBusy 的回调。
  ///
  /// return TUIResult
  Future<TUIResult> ignore() async {
    return await TUICallEnginePlatform.instance.ignore();
  }

  /// 邀请用户加入此次群组通话。
  /// 使用场景：一个群组通话中的用户主动邀请其他人时使用。
  ///
  /// @param userIdList 目标用户的 userId 列表
  /// @param params     通话参数扩展字段，例如：离线推送自定义内容
  /// return TUIResult
  Future<void> iniviteUser(List<String> userIdList, TUICallParams params,
      TUIValueCallback callback) async {
    return await TUICallEnginePlatform.instance
        .iniviteUser(userIdList, params, callback);
  }

  /// 主动加入此次群组通话。
  /// 使用场景：群组内用户主动加入此次群组通话使用。
  ///
  /// @param roomId        此次通话的音视频房间 ID
  /// @param groupId       此次群组通话的群 ID
  /// @param callMediaType 通话的媒体类型，例如：视频通话、语音通话
  /// return TUIResult
  Future<TUIResult> joinInGroupCall(
      TUIRoomId roomId, String groupId, TUICallMediaType mediaType) async {
    return await TUICallEnginePlatform.instance
        .joinInGroupCall(roomId, groupId, mediaType);
  }

  /// 切换通话媒体类型，比如视频通话切音频通话
  ///
  /// @param newType 通话的媒体类型，例如：视频通话、语音通话
  Future<void> switchCallMediaType(TUICallMediaType mediaType) async {
    await TUICallEnginePlatform.instance.switchCallMediaType(mediaType);
  }

  /// 开始订阅远端用户视频流
  ///
  /// @param userId    目标用户的 userId
  /// @param viewId    对应native层的TRTCCloudVideoView
  Future<void> startRemoteView(String userId, int? viewId) async {
    await TUICallEnginePlatform.instance.startRemoteView(userId, viewId);
  }

  /// 停止订阅远端用户视频流
  ///
  /// @param userId 目标用户的 userId
  Future<void> stopRemoteView(String userId) async {
    await TUICallEnginePlatform.instance.stopRemoteView(userId);
  }

  /// 开启摄像头
  ///
  /// @param camera    前置/后置 摄像头
  /// @param viewId    对应native层的TRTCCloudVideoView
  /// return TUIResult
  Future<TUIResult> openCamera(TUICamera camera, int? viewId) async {
    return await TUICallEnginePlatform.instance.openCamera(camera, viewId);
  }

  /// 关闭摄像头
  Future<void> closeCamera() async {
    await TUICallEnginePlatform.instance.closeCamera();
  }

  /// 切换前后摄像头
  ///
  /// @param camera 前置/后置 摄像头
  Future<void> switchCamera(TUICamera camera) async {
    await TUICallEnginePlatform.instance.switchCamera(camera);
  }

  /// 打开麦克风
  ///
  /// return TUIResult
  Future<TUIResult> openMicrophone() async {
    return await TUICallEnginePlatform.instance.openMicrophone();
  }

  /// 关闭麦克风
  Future<void> closeMicrophone() async {
    await TUICallEnginePlatform.instance.closeMicrophone();
  }

  /// 选择音频播放设备
  ///
  /// @param device 听筒/扬声器
  Future<void> selectAudioPlaybackDevice(TUIAudioPlaybackDevice device) async {
    await TUICallEnginePlatform.instance.selectAudioPlaybackDevice(device);
  }

  /// 设置用户的昵称、头像
  ///
  /// @param nickname 用户昵称
  /// @param avatar   用户头像（格式为 URL）
  /// return TUIResult
  Future<TUIResult> setSelfInfo(String nickname, String avatar) async {
    return await TUICallEnginePlatform.instance.setSelfInfo(nickname, avatar);
  }

  /// 开启/关闭 TUICallEngine 的多设备登录模式
  ///
  /// @param enable   开启:YES; 关闭:NO
  /// return TUIResult
  Future<TUIResult> enableMultiDeviceAbility(bool enable) async {
    return await TUICallEnginePlatform.instance
        .enableMultiDeviceAbility(enable);
  }

  /// 查询通话记录
  ///
  /// @param filter 查询过滤条件
  /// @param succ 查询成功的结果事件
  /// return TUIResult
  Future<void> queryRecentCalls(
      TUICallRecentCallsFilter filter, TUIValueCallback callback) async {
    return await TUICallEnginePlatform.instance
        .queryRecentCalls(filter, callback);
  }

  /// 删除通话记录
  ///
  /// @param callIdList 待删除记录的callId列表
  /// @param succ 删除成功的callId列表
  /// return TUIResult
  Future<void> deleteRecordCalls(
      List<String> callIdList, TUIValueCallback callback) async {
    return await TUICallEnginePlatform.instance
        .deleteRecordCalls(callIdList, callback);
  }

  /// 设置美颜级别
  ///
  /// @param level    美颜级别
  /// return TUIResult
  Future<TUIResult> setBeautyLevel(double level) async {
    return await TUICallEnginePlatform.instance.setBeautyLevel(level);
  }

  /// 调用实验性接口
  ///
  /// @param jsonObject
  Future<void> callExperimentalAPI(Map<String, dynamic> jsonMap) async {
    await TUICallEnginePlatform.instance.callExperimentalAPI(jsonMap);
  }
}
